import tempfile
import pytest
from pathlib import Path
from pydantic import ValidationError

from gama_config.gama_gs_io import GsConfigIO
from gama_config.gama_gs import GamaGsConfig, Mode, LogLevel, Network


class TestGsConfigIO:
    def test_get_path_default_directory(self):
        """Test that get_path returns correct default path."""
        config_io = GsConfigIO()
        expected_path = Path("~/.config/greenroom/gama_gs.yml").expanduser()
        assert config_io.get_path() == expected_path

    def test_get_path_custom_directory(self):
        """Test that get_path returns correct path with custom directory."""
        config_io = GsConfigIO("custom/subdir")
        expected_path = Path("~/.config/greenroom/custom/subdir/gama_gs.yml").expanduser()
        assert config_io.get_path() == expected_path

    def test_parse_empty_config(self):
        """Test parsing empty configuration dictionary."""
        config_io = GsConfigIO()
        result = config_io.parse({})
        assert isinstance(result, GamaGsConfig)

    def test_parse_config_with_values(self):
        """Test parsing configuration dictionary with specific values."""
        config_io = GsConfigIO()
        config_dict = {
            "vessel_ip": None,
            "log_level": "info",
            "mode": "none",
            "network": "shared",
            "prod": False,
        }
        result = config_io.parse(config_dict)
        assert result.log_level == LogLevel.INFO
        assert result.mode == Mode.NONE
        assert result.network == Network.SHARED
        assert result.prod is False

    def test_parse_none_config(self):
        """Test parsing None configuration."""
        config_io = GsConfigIO()
        result = config_io.parse({})
        assert isinstance(result, GamaGsConfig)

    def test_write_and_read_config(self):
        """Test writing and reading configuration to/from temporary directory."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = GsConfigIO(temp_dir)

            test_config = GamaGsConfig(
                log_level=LogLevel.INFO,
                mode=Mode.NONE,
                network=Network.SHARED,
                prod=False,
            )

            config_io.write(test_config)

            config_path = config_io.get_path()
            assert config_path.exists()

            read_config = config_io.read()

            assert read_config.log_level == LogLevel.INFO
            assert read_config.mode == Mode.NONE
            assert read_config.network == Network.SHARED
            assert read_config.prod is False

    def test_write_creates_parent_directory(self):
        """Test that write creates parent directory if it doesn't exist."""
        with tempfile.TemporaryDirectory() as temp_dir:
            nested_path = f"{temp_dir}/nested/subdir"
            config_io = GsConfigIO(nested_path)

            test_config = GamaGsConfig(mode=Mode.NONE)

            config_io.write(test_config)

            config_path = config_io.get_path()
            assert config_path.exists()
            assert config_path.parent.exists()

    def test_write_includes_schema_header(self):
        """Test that written file includes YAML schema header."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = GsConfigIO(temp_dir)
            test_config = GamaGsConfig(mode=Mode.NONE)

            config_io.write(test_config)

            config_path = config_io.get_path()
            with open(config_path) as f:
                content = f.read()

            expected_header = f"# yaml-language-server: $schema={GsConfigIO.schema_url}"
            assert content.startswith(expected_header)

    def test_read_nonexistent_file_raises_error(self):
        """Test that reading a non-existent file raises FileNotFoundError."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = GsConfigIO(f"{temp_dir}/nonexistent")

            with pytest.raises(FileNotFoundError, match="Could not find config file"):
                config_io.read()

    def test_parse_invalid_mode_raises_error(self):
        """Test that parsing invalid mode raises ValidationError."""
        config_io = GsConfigIO()
        config_dict = {
            "vessel_ip": None,
            "log_level": "info",
            "mode": "goblin",
            "network": "shared",
            "prod": False,
        }
        with pytest.raises(ValidationError, match="Input should be.*logitech_extreme"):
            config_io.parse(config_dict)

    def test_parse_invalid_network_raises_error(self):
        """Test that parsing invalid network raises ValidationError."""
        config_io = GsConfigIO()
        config_dict = {
            "vessel_ip": None,
            "log_level": "info",
            "mode": "none",
            "network": "starlink",
            "prod": False,
        }
        with pytest.raises(ValidationError, match="Input should be.*host"):
            config_io.parse(config_dict)



# IMPORTANT
# After changing this file, run `python3 -m gama_config.generate_schemas`
# To re-generate the json schemas

import os
from enum import Enum
from functools import lru_cache

from typing import Any, List, Union, Literal, Optional
from gama_config import LogLevel
from greenstream_config.types import Camera
from pydantic import Field, BaseModel, ConfigDict, RootModel, field_validator

DEFAULT_NAMESPACE_VESSEL = "vessel_1"


class Mode(str, Enum):
    SIMULATOR = "simulator"
    HARDWARE = "hardware"
    STUBS = "stubs"
    HITL_SIMULATOR = "hitl_simulator"

    def __str__(self):
        return self.value


class Network(str, Enum):
    SHARED = "shared"
    HOST = "host"

    def __str__(self):
        return self.value


class Variant(str, Enum):
    WHISKEY_BRAVO = "whiskey_bravo"
    EDUCAT = "educat"
    ORACLE_2_2 = "oracle_2_2"
    ORACLE_22 = "oracle_22"
    ARMIDALE = "armidale"
    WAVEFLYER = "waveflyer"
    DMAK = "dmak"
    MARS = "mars"
    FREMANTLE = "fremantle"
    BLUE_BOAT = "blue_boat"
    TENGGARA = "tenggara"
    TACK = "tack"
    AUSTAL_M_USV = "austal_m_usv"

    def __str__(self):
        return self.value


class SubVariant(str, Enum):
    M_USV = "m_usv"
    RHIB = "rhib"

    def __str__(self):
        return self.value


class AutopilotType(str, Enum):
    STANDARD_AUTOPILOT = "standard_autopilot"
    PREDICTIVE_AUTOPILOT = "predictive_autopilot"

    def __str__(self):
        return self.value


class DiscoverySimple(BaseModel):
    type: Literal["simple"] = "simple"
    ros_domain_id: int = Field(
        default=0,
        description="ROS domain ID",
    )
    own_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface address of the primary network interface. This is where DDS traffic will route to.",
    )
    discovery_range: Literal["subnet", "localhost"] = Field(
        default="localhost",
        description="Discovery range: 'localhost' sets ROS_AUTOMATIC_DISCOVERY_RANGE to LOCALHOST, 'subnet' sets it to SUBNET.",
    )


class DiscoveryFastDDS(BaseModel):
    type: Literal["fastdds"] = "fastdds"
    with_discovery_server: bool = Field(
        default=True, description="Run the discovery server. It will bind to 0.0.0.0:11811"
    )
    discovery_server_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface of the discovery server. Assumes port of 11811",
    )
    own_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface address of the primary network interface. This is where DDS traffic will route to.",
    )


class DiscoveryZenoh(BaseModel):
    type: Literal["zenoh"] = "zenoh"
    with_discovery_server: bool = Field(default=True, description="Run the zenoh router")
    discovery_server_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface of the discovery server.",
    )


class Ports(BaseModel):
    host: str = Field(
        default="http://localhost",
        description="Host for the GAMA services. This will tell other apps where to find GAMA. It should be the hostname or IP of the machine running GAMA.",
    )
    ui: int = Field(
        default=3000,
        description="Port for the GAMA UI. Note, the ENV server will run on this + 100",
    )
    chart_tiler: int = Field(default=3001, description="Port for the Chart Tiler service")
    chart_api: int = Field(default=3002, description="Port for the Chart API service")
    docs: int = Field(default=3003, description="Port for the GAMA documentation server")
    mission_plan_runner: int = Field(
        default=3004, description="Port for the Mission Plan Runner service"
    )
    greenstream_signalling: int = Field(
        default=3005, description="Port for the Greenstream signalling server"
    )
    greenstream_ui: int = Field(default=3006, description="Port for the Greenstream UI")
    tapedeck: int = Field(default=3007, description="Port for the Tapedeck service")
    data_bridge: int = Field(default=3010, description="Port for the Data Bridge service")
    gs_data_bridge: int = Field(
        default=3011, description="Port for the Ground Station Data Bridge service"
    )


Discovery = Union[DiscoveryZenoh, DiscoveryFastDDS, DiscoverySimple]


class GamaVesselConfig(BaseModel):
    model_config = ConfigDict(
        json_encoders={
            Variant: lambda v: v.value,
            SubVariant: lambda v: v.value,
            Mode: lambda v: v.value,
            LogLevel: lambda v: v.value,
            Network: lambda v: v.value,
        },
    )

    variant: Variant
    display_name: str = "GAMA Vessel"
    namespace_vessel: str = DEFAULT_NAMESPACE_VESSEL
    namespace_groundstation: str = "groundstation"
    mode: Mode = Mode.SIMULATOR
    network: Network = Network.HOST
    prod: bool = False
    log_level: LogLevel = LogLevel.INFO
    cameras: Optional[List[Camera]] = None
    record: bool = False
    advanced_configuration: Optional[dict[str, str]] = None
    components: Any = Field(default_factory=dict)
    log_directory: str = "~/greenroom/gama/logs"
    recording_directory: str = "~/greenroom/gama/recordings"
    charts_directory: str = "~/greenroom/charts"
    discovery: Discovery = Field(
        default_factory=DiscoverySimple,
        discriminator="type",
    )
    autopilot_types: List[AutopilotType] = Field(
        default=[AutopilotType.STANDARD_AUTOPILOT],
    )
    ports: Ports = Field(
        default_factory=Ports,
        description="Ports configuration for various GAMA services",
    )

    @field_validator("autopilot_types")
    def ensure_non_empty_list(cls, autopilot_types_list):
        if not autopilot_types_list:
            raise ValueError("autopilot_types must contain at least one value")
        return autopilot_types_list


class ArmidaleVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.ARMIDALE] = Variant.ARMIDALE


class EducatVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.EDUCAT] = Variant.EDUCAT


class Oracle22VesselConfig(GamaVesselConfig):
    variant: Literal[Variant.ORACLE_22] = Variant.ORACLE_22


class Oracle2_2VesselConfig(GamaVesselConfig):
    variant: Literal[Variant.ORACLE_2_2] = Variant.ORACLE_2_2


class WaveflyerVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.WAVEFLYER] = Variant.WAVEFLYER


class WhiskeyBravoVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.WHISKEY_BRAVO] = Variant.WHISKEY_BRAVO


class DMAKVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.DMAK] = Variant.DMAK


class MarsVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.MARS] = Variant.MARS


class FremantleVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.FREMANTLE] = Variant.FREMANTLE


class BlueBoatVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.BLUE_BOAT] = Variant.BLUE_BOAT


class TenggaraVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.TENGGARA] = Variant.TENGGARA


class TackVesselConfig(GamaVesselConfig):
    variant: Literal[Variant.TACK] = Variant.TACK


class Austal_M_USV(GamaVesselConfig):
    variant: Literal[Variant.AUSTAL_M_USV] = Variant.AUSTAL_M_USV
    sub_variant: SubVariant = Field(
        default=SubVariant.M_USV, description="Sub-variant configuration for Austal M_USV vessel"
    )


VariantVesselConfig = Union[
    ArmidaleVesselConfig,
    EducatVesselConfig,
    Oracle22VesselConfig,
    Oracle2_2VesselConfig,
    WaveflyerVesselConfig,
    WhiskeyBravoVesselConfig,
    DMAKVesselConfig,
    MarsVesselConfig,
    FremantleVesselConfig,
    BlueBoatVesselConfig,
    TenggaraVesselConfig,
    TackVesselConfig,
    Austal_M_USV,
]

DEFAULT_VARIANT_CONFIGS_MAP: dict[Variant, VariantVesselConfig] = {
    Variant.ARMIDALE: ArmidaleVesselConfig(),
    Variant.EDUCAT: EducatVesselConfig(),
    Variant.ORACLE_22: Oracle22VesselConfig(),
    Variant.ORACLE_2_2: Oracle2_2VesselConfig(),
    Variant.WAVEFLYER: WaveflyerVesselConfig(),
    Variant.WHISKEY_BRAVO: WhiskeyBravoVesselConfig(),
    Variant.DMAK: DMAKVesselConfig(),
    Variant.MARS: MarsVesselConfig(),
    Variant.FREMANTLE: FremantleVesselConfig(),
    Variant.BLUE_BOAT: BlueBoatVesselConfig(),
    Variant.TENGGARA: TenggaraVesselConfig(),
    Variant.TACK: TackVesselConfig(),
    Variant.AUSTAL_M_USV: Austal_M_USV(),
}


class VariantVesselConfigRoot(RootModel):
    root: VariantVesselConfig = Field(..., discriminator="variant")


def get_vessel_config_io():
    """Get a VesselConfigIO instance using the GAMA_CONFIG_DIR environment variable."""
    from gama_config.gama_vessel_io import VesselConfigIO

    config_dir = os.environ.get("GAMA_CONFIG_DIR")
    if config_dir is None:
        raise ValueError("GAMA_CONFIG_DIR environment variable is not set.")
    return VesselConfigIO(config_directory=config_dir)


@lru_cache(maxsize=1)
def get_vessel_config_instance() -> VariantVesselConfig:
    """Get the current vessel configuration (singleton)."""
    return get_vessel_config_io().read()



import yaml
import os
from typing import Any
from pathlib import Path

from gama_config.helpers import YamlDumper
from gama_config.gama_vessel import VariantVesselConfig, VariantVesselConfigRoot


class VesselConfigIO:
    """Configuration I/O handler for GAMA Vessel configuration files."""

    file_name = "gama_vessel.yml"
    schema_url = "https://greenroom-robotics.github.io/gama/schemas/gama_vessel.schema.json"

    def __init__(self, config_directory: str | Path = ""):
        """Initialize VesselConfigIO with a configurable config directory.

        Args:
            config_directory: Base configuration directory path. Defaults to ~/.config/greenroom
        """
        if str(config_directory).startswith("~"):
            self.config_directory = Path(config_directory).expanduser()
        else:
            self.config_directory = (
                Path("~/.config/greenroom").joinpath(config_directory).expanduser()
            )

    def get_path(self) -> Path:
        """Returns the full path to the vessel configuration file."""
        return self.config_directory / self.file_name

    def parse(self, config: dict[str, Any]) -> VariantVesselConfig:
        """Parse a configuration dictionary into a VariantVesselConfig object."""
        return VariantVesselConfigRoot(root=config or {}).root  # type: ignore[return-value]

    def read(self) -> VariantVesselConfig:
        """Read and parse the vessel configuration file."""
        path = self.get_path()
        try:
            with open(path) as stream:
                return self.parse(yaml.safe_load(stream))
        except FileNotFoundError:
            raise FileNotFoundError(f"Could not find config file: {path}")
        except Exception as e:
            raise ValueError(f"Could not parse config file {path} - {e}")

    def write(self, config: VariantVesselConfig, include_defaults: bool = False):
        """Write a VariantVesselConfig object to the configuration file."""
        path = self.get_path()
        # Make the parent dir if it doesn't exist
        os.makedirs(path.parent, exist_ok=True)

        if include_defaults:
            json_string = config.model_dump(mode="json")
        else:
            json_string = config.model_dump(mode="json", exclude_defaults=True)
            # Always include variant field as it's required for discriminated union parsing
            if "variant" not in json_string:
                json_string["variant"] = config.variant.value
        with open(path, "w") as stream:
            print(f"Writing: {path}")
            headers = f"# yaml-language-server: $schema={VesselConfigIO.schema_url}"
            data = "\n".join([headers, yaml.dump(json_string, Dumper=YamlDumper, sort_keys=True)])
            stream.write(data)

    def serialise(self, config: VariantVesselConfig) -> str:
        """Serialize a VariantVesselConfig object to YAML string."""
        json_data = config.model_dump(mode="json", exclude_defaults=True)
        # Always include variant field as it's required for discriminated union parsing
        if "variant" not in json_data:
            json_data["variant"] = config.variant.value
        return yaml.dump(json_data, default_flow_style=True, sort_keys=True)



import tempfile
import pytest
from pathlib import Path
from pydantic import ValidationError

from gama_config.gama_vessel_io import VesselConfigIO
from gama_config.gama_vessel import (
    WaveflyerVesselConfig,
    Mode,
    Variant,
    LogLevel,
    Network,
    EducatVesselConfig,
)


class TestVesselConfigIO:
    def test_get_path_default_directory(self):
        """Test that get_path returns correct default path."""
        config_io = VesselConfigIO()
        expected_path = Path("~/.config/greenroom/gama_vessel.yml").expanduser()
        assert config_io.get_path() == expected_path

    def test_get_path_custom_directory(self):
        """Test that get_path returns correct path with custom directory."""
        config_io = VesselConfigIO("custom/subdir")
        expected_path = Path("~/.config/greenroom/custom/subdir/gama_vessel.yml").expanduser()
        assert config_io.get_path() == expected_path

    def test_parse_empty_config_raises_error(self):
        """Test parsing empty configuration dictionary raises ValidationError."""
        config_io = VesselConfigIO()
        with pytest.raises(
            ValidationError, match="Unable to extract tag using discriminator 'variant'"
        ):
            config_io.parse({})

    def test_parse_educat_config_with_values(self):
        """Test parsing configuration dictionary with educat variant."""
        config_io = VesselConfigIO()
        config_dict = {
            "variant": "educat",
            "namespace_vessel": "vessel_1",
            "namespace_groundstation": "groundstation",
            "mode": "simulator",
            "network": "host",
            "prod": False,
            "log_level": "info",
            "ubiquity_user": "",
            "ubiquity_pass": "",
            "ubiquity_ip": "",
            "cameras": None,
            "record": False,
            "components": {
                "autopilot": {"pid": [1, 2, 3]},
            },
            "advanced_configuration": {"first_launch_arg": "value", "second_launch_arg": "value"},
        }
        result = config_io.parse(config_dict)
        assert isinstance(result, EducatVesselConfig)
        assert result.namespace_vessel == "vessel_1"
        assert result.namespace_groundstation == "groundstation"
        assert result.mode == Mode.SIMULATOR
        assert result.network == Network.HOST
        assert result.prod is False
        assert result.log_level == LogLevel.INFO
        assert result.cameras is None
        assert result.record is False
        assert result.components == {"autopilot": {"pid": [1, 2, 3]}}
        assert result.advanced_configuration == {
            "first_launch_arg": "value",
            "second_launch_arg": "value",
        }

    def test_parse_waveflyer_config(self):
        """Test parsing configuration dictionary with waveflyer variant."""
        config_io = VesselConfigIO()
        config_dict = {
            "variant": "waveflyer",
            "namespace_vessel": "vessel_1",
            "namespace_groundstation": "groundstation",
            "mode": "simulator",
            "network": "host",
            "prod": False,
            "log_level": "info",
            "ubiquity_user": "",
            "ubiquity_pass": "",
            "ubiquity_ip": "",
            "cameras": None,
            "record": False,
            "advanced_configuration": {"random_launch_arg": "value"},
        }
        result = config_io.parse(config_dict)
        assert isinstance(result, WaveflyerVesselConfig)
        assert result.variant == Variant.WAVEFLYER
        assert result.namespace_vessel == "vessel_1"
        assert result.namespace_groundstation == "groundstation"
        assert result.mode == Mode.SIMULATOR
        assert result.network == Network.HOST
        assert result.prod is False
        assert result.log_level == LogLevel.INFO
        assert result.cameras is None
        assert result.record is False
        assert result.components == {}
        assert result.advanced_configuration == {"random_launch_arg": "value"}

    def test_write_and_read_educat_config(self):
        """Test writing and reading educat configuration to/from temporary directory."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = VesselConfigIO(temp_dir)

            test_config = EducatVesselConfig(
                namespace_vessel="test_vessel",
                namespace_groundstation="test_gs",
                mode=Mode.SIMULATOR,
                network=Network.HOST,
                prod=False,
                log_level=LogLevel.INFO,
                cameras=None,
                record=False,
                components={"autopilot": {"pid": [1, 2, 3]}},
                advanced_configuration={"test_arg": "value"},
            )

            config_io.write(test_config)

            config_path = config_io.get_path()
            assert config_path.exists()

            read_config = config_io.read()

            assert isinstance(read_config, EducatVesselConfig)
            assert read_config.namespace_vessel == "test_vessel"
            assert read_config.namespace_groundstation == "test_gs"
            assert read_config.mode == Mode.SIMULATOR
            assert read_config.network == Network.HOST
            assert read_config.prod is False
            assert read_config.log_level == LogLevel.INFO
            assert read_config.cameras is None
            assert read_config.record is False
            assert read_config.components == {"autopilot": {"pid": [1, 2, 3]}}
            assert read_config.advanced_configuration == {"test_arg": "value"}

    def test_write_creates_parent_directory(self):
        """Test that write creates parent directory if it doesn't exist."""
        with tempfile.TemporaryDirectory() as temp_dir:
            nested_path = f"{temp_dir}/nested/subdir"
            config_io = VesselConfigIO(nested_path)

            test_config = EducatVesselConfig()

            config_io.write(test_config)

            config_path = config_io.get_path()
            assert config_path.exists()
            assert config_path.parent.exists()

    def test_write_includes_schema_header(self):
        """Test that written file includes YAML schema header."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = VesselConfigIO(temp_dir)
            test_config = EducatVesselConfig()

            config_io.write(test_config)

            config_path = config_io.get_path()
            with open(config_path) as f:
                content = f.read()

            expected_header = f"# yaml-language-server: $schema={VesselConfigIO.schema_url}"
            assert content.startswith(expected_header)

    def test_read_nonexistent_file_raises_error(self):
        """Test that reading a non-existent file raises FileNotFoundError."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = VesselConfigIO(f"{temp_dir}/nonexistent")

            with pytest.raises(FileNotFoundError, match="Could not find config file"):
                config_io.read()

    def test_parse_invalid_mode_raises_error(self):
        """Test that parsing invalid mode raises ValidationError."""
        config_io = VesselConfigIO()
        config_dict = {
            "cameras": None,
            "log_level": "info",
            "mode": "goblin",
            "network": "host",
            "prod": False,
            "ubiquity_ip": "",
            "ubiquity_pass": "",
            "ubiquity_user": "",
            "variant": "educat",
        }
        with pytest.raises(ValidationError, match="Input should be.*hitl_simulator"):
            config_io.parse(config_dict)

    def test_parse_invalid_variant_raises_error(self):
        """Test that parsing invalid variant raises ValidationError."""
        config_io = VesselConfigIO()
        config_dict = {
            "cameras": None,
            "log_level": "info",
            "mode": "stubs",
            "network": "host",
            "prod": False,
            "ubiquity_ip": "",
            "ubiquity_pass": "",
            "ubiquity_user": "",
            "variant": "killer-robot",
        }
        with pytest.raises(
            ValidationError,
            match="Input tag 'killer-robot'.*does not match any of the expected tags",
        ):
            config_io.parse(config_dict)

    def test_parse_invalid_camera_type_raises_error(self):
        """Test that parsing invalid camera type raises ValidationError."""
        config_io = VesselConfigIO()
        config_dict = {
            "cameras": [
                {
                    "name": "bow",
                    "order": 0,
                    "type": 89,  # Should be string
                }
            ],
            "log_level": "info",
            "mode": "stubs",
            "network": "host",
            "prod": False,
            "ubiquity_ip": "",
            "ubiquity_pass": "",
            "ubiquity_user": "",
            "variant": "educat",
        }
        with pytest.raises(ValidationError, match="Input should be a valid string"):
            config_io.parse(config_dict)

    def test_write_excludes_default_values(self):
        """Test that written config excludes fields with default values."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = VesselConfigIO(temp_dir)

            # Create a config with only variant set (all other fields will use defaults)
            test_config = EducatVesselConfig()

            config_io.write(test_config)

            config_path = config_io.get_path()
            with open(config_path) as f:
                content = f.read()

            # Verify default port values are not in the YAML output
            assert "ui: 3000" not in content
            assert "chart_tiler: 3001" not in content
            assert "chart_api: 3002" not in content

            # Verify default other values are not in the YAML output
            assert "display_name: GAMA Vessel" not in content
            assert "namespace_vessel: vessel_1" not in content
            assert "mode: simulator" not in content
            assert "network: host" not in content
            assert "prod: false" not in content
            assert "log_level: info" not in content

            # Verify only the variant is present (as it's required)
            assert "variant: educat" in content

    def test_serialise_excludes_default_values(self):
        """Test that serialise method excludes fields with default values."""
        config_io = VesselConfigIO()
        test_config = EducatVesselConfig()

        serialised = config_io.serialise(test_config)

        # Verify default port values are not in the serialized output
        assert "ui: 3000" not in serialised
        assert "chart_tiler: 3001" not in serialised
        assert "chart_api: 3002" not in serialised

        # Verify default other values are not in the serialized output
        assert "display_name: GAMA Vessel" not in serialised
        assert "namespace_vessel: vessel_1" not in serialised
        assert "mode: simulator" not in serialised
        assert "network: host" not in serialised
        assert "prod: false" not in serialised
        assert "log_level: info" not in serialised

        # Verify only the variant is present (as it's required)
        assert "variant: educat" in serialised

    def test_write_with_include_defaults_flag(self):
        """Test that written config includes defaults when include_defaults=True."""
        with tempfile.TemporaryDirectory() as temp_dir:
            config_io = VesselConfigIO(temp_dir)

            # Create a config with defaults
            test_config = EducatVesselConfig()

            # Write with include_defaults=True
            config_io.write(test_config, include_defaults=True)

            config_path = config_io.get_path()
            with open(config_path) as f:
                content = f.read()

            # Verify default port values ARE included in the YAML output
            assert "ui: 3000" in content
            assert "chart_tiler: 3001" in content
            assert "chart_api: 3002" in content

            # Verify default other values ARE included in the YAML output
            assert "display_name: GAMA Vessel" in content
            assert "namespace_vessel: vessel_1" in content
            assert "mode: simulator" in content
            assert "network: host" in content
            assert "prod: false" in content
            assert "log_level: info" in content

            # Verify variant is still present
            assert "variant: educat" in content



import yaml
from typing import Any, Callable


class YamlDumper(yaml.Dumper):
    """
    A YAML dumpler that show lists on the same line if they do not contain dicts or list
    """

    def represent_sequence(self, tag, sequence, flow_style=None):
        if isinstance(sequence, list) and all(
            [not isinstance(item, (dict, list)) for item in sequence]
        ):
            flow_style = True
        return super().represent_sequence(tag, sequence, flow_style)

    def represent_mapping(self, tag, mapping, flow_style=None):
        flow_style = False
        return super().represent_mapping(tag, mapping, flow_style)



import yaml
import os
from typing import Any
from pathlib import Path

from gama_config.helpers import YamlDumper
from gama_config.gama_gs import GamaGsConfig


class GsConfigIO:
    """Configuration I/O handler for GAMA Ground Station configuration files."""

    file_name = "gama_gs.yml"
    schema_url = "https://greenroom-robotics.github.io/gama/schemas/gama_gs.schema.json"

    def __init__(self, config_directory: str | Path = ""):
        """Initialize GsConfigIO with a configurable config directory.

        Args:
            config_directory: Base configuration directory path. Defaults to ~/.config/greenroom
        """
        if str(config_directory).startswith("~"):
            self.config_directory = Path(config_directory).expanduser()
        else:
            self.config_directory = (
                Path("~/.config/greenroom").joinpath(config_directory).expanduser()
            )

    def get_path(self) -> Path:
        """Returns the full path to the ground station configuration file."""
        return self.config_directory / self.file_name

    def parse(self, config: dict[str, Any]) -> GamaGsConfig:
        """Parse a configuration dictionary into a GamaGsConfig object."""
        return GamaGsConfig(**config or {})

    def read(self) -> GamaGsConfig:
        """Read and parse the ground station configuration file."""
        path = self.get_path()
        try:
            with open(path) as stream:
                return self.parse(yaml.safe_load(stream))
        except FileNotFoundError:
            raise FileNotFoundError(f"Could not find config file: {path}")
        except Exception as e:
            raise ValueError(f"Could not parse config file {path} - {e}")

    def write(self, config: GamaGsConfig):
        """Write a GamaGsConfig object to the configuration file."""
        path = self.get_path()
        # Make the parent dir if it doesn't exist
        os.makedirs(path.parent, exist_ok=True)
        json_string = config.model_dump(mode="json")
        with open(path, "w") as stream:
            print(f"Writing: {path}")
            headers = f"# yaml-language-server: $schema={GsConfigIO.schema_url}"
            data = "\n".join([headers, yaml.dump(json_string, Dumper=YamlDumper, sort_keys=True)])
            stream.write(data)

    def serialise(self, config: GamaGsConfig) -> str:
        """Serialize a GamaGsConfig object to YAML string."""
        return yaml.dump(config.model_dump_json(), default_flow_style=True, sort_keys=True)



# IMPORTANT
# After changing this file, run `python3 -m gama_config.generate_schemas`
# To re-generate the json schemas

import os
from pydantic import BaseModel, Field
from enum import Enum
from typing import Literal, Union
from gama_config import LogLevel

DEFAULT_NAMESPACE_GROUNDSTATION = "groundstation"


class Mode(str, Enum):
    NONE = "none"
    XBOX = "xbox"
    XBOX_SERIES_X = "xbox_series_x"
    THRUSTMASTER = "thrustmaster"
    THRUSTMASTER_COMBO = "thrustmaster_combo"
    WARTHOG = "warthog"
    WARTHOG_COMBO = "warthog_combo"
    AERONAV = "aeronav"
    SINGLE_UNKNOWN = "single_unknown"
    DUAL_UNKNOWN = "dual_unknown"
    GLADIATOR = "gladiator"
    LOGITECH_EXTREME = "logitech_extreme"
    PLAYSTATION_DUAL_SENSE = "playstation_dual_sense"


class Network(str, Enum):
    SHARED = "shared"
    HOST = "host"


class DiscoverySimple(BaseModel):
    type: Literal["simple"] = "simple"
    ros_domain_id: int = Field(
        default=0,
        description="ROS domain ID",
    )
    own_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface address of the primary network interface. This is where DDS traffic will route to.",
    )
    discovery_range: Literal["subnet", "localhost"] = Field(
        default="localhost",
        description="Discovery range: 'localhost' sets ROS_AUTOMATIC_DISCOVERY_RANGE to LOCALHOST, 'subnet' sets it to SUBNET.",
    )


class DiscoveryFastDDS(BaseModel):
    type: Literal["fastdds"] = "fastdds"
    with_discovery_server: bool = Field(
        default=True, description="Run the discovery server. It will bind to 0.0.0.0:11811"
    )
    discovery_server_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface of the discovery server. Assumes port of 11811",
    )
    own_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface address of the primary network interface. This is where DDS traffic will route to.",
    )


class DiscoveryZenoh(BaseModel):
    type: Literal["zenoh"] = "zenoh"
    with_discovery_server: bool = Field(default=True, description="Run the zenoh router")
    discovery_server_ip: str = Field(
        default="0.0.0.0",
        description="IP/host/interface of the discovery server. Assumes port of 7447",
    )


Discovery = Union[DiscoveryZenoh, DiscoveryFastDDS, DiscoverySimple]


class GamaGsConfig(BaseModel):
    namespace_vessel: str = "vessel_1"
    namespace_groundstation: str = DEFAULT_NAMESPACE_GROUNDSTATION
    mode: Mode = Mode.NONE
    buttons: bool = False
    network: Network = Network.SHARED
    prod: bool = False
    log_level: LogLevel = LogLevel.INFO
    remote_cmd_override: bool = False
    discovery: Discovery = Field(
        default_factory=DiscoverySimple,
        discriminator="type",
    )
    ui: bool = False


def get_gs_config_io():
    """Get a GsConfigIO instance using the GAMA_CONFIG_DIR environment variable."""
    from gama_config.gama_gs_io import GsConfigIO

    config_dir = os.environ.get("GAMA_CONFIG_DIR")
    if config_dir is None:
        raise ValueError("GAMA_CONFIG_DIR environment variable is not set.")
    return GsConfigIO(config_directory=config_dir)
